==> python-language-23.1.1-sources.jar: com/oracle/graal/python/builtins/objects/cext/hpy/GraalHPyTracker.java <==
/*
 * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.cext.hpy;

import java.util.Arrays;

import com.oracle.graal.python.builtins.objects.cext.hpy.GraalHPyNodes.HPyCloseHandleNode;
import com.oracle.graal.python.util.OverflowException;
import com.oracle.graal.python.util.PythonUtils;
import com.oracle.truffle.api.CompilerDirectives.TruffleBoundary;

public final class GraalHPyTracker {
    private static final int HPYTRACKER_INITIAL_SIZE = 5;

    private GraalHPyHandle[] handles;
    private int cursor;

    public GraalHPyTracker(int capacity) {
        int size = capacity == 0 ? HPYTRACKER_INITIAL_SIZE : capacity;
        this.handles = new GraalHPyHandle[size];
    }

    public void add(GraalHPyHandle h) throws OverflowException {
        handles[cursor++] = h;
        if (handles.length <= cursor) {
            resize();
        }
    }

    @TruffleBoundary
    private void resize() throws OverflowException {
        handles = Arrays.copyOf(handles, PythonUtils.multiplyExact(handles.length, 2) - 1);
    }

    public void free(HPyCloseHandleNode closeHandleNode) {
        assert cursor <= handles.length;
        for (int i = 0; i < cursor; i++) {
            closeHandleNode.execute(handles[i]);
        }
        cursor = 0;
    }

    public void removeAll() {
        for (int i = 0; i < handles.length; i++) {
            handles[i] = null;
        }
        cursor = 0;
    }
}

==> python-language-24.0.2-sources.jar: com/oracle/graal/python/builtins/objects/cext/hpy/GraalHPyTracker.java <==
/*
 * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.graal.python.builtins.objects.cext.hpy;

import java.util.Arrays;

import com.oracle.graal.python.builtins.objects.cext.hpy.GraalHPyNodes.HPyCloseHandleNode;
import com.oracle.graal.python.util.OverflowException;
import com.oracle.graal.python.util.PythonUtils;
import com.oracle.truffle.api.CompilerDirectives.TruffleBoundary;
import com.oracle.truffle.api.nodes.Node;

public final class GraalHPyTracker {
    private static final int HPYTRACKER_INITIAL_SIZE = 5;

    private GraalHPyHandle[] handles;
    private int cursor;

    public GraalHPyTracker(int capacity) {
        int size = capacity == 0 ? HPYTRACKER_INITIAL_SIZE : capacity;
        this.handles = new GraalHPyHandle[size];
    }

    public void add(GraalHPyHandle h) throws OverflowException {
        handles[cursor++] = h;
        if (handles.length <= cursor) {
            resize();
        }
    }

    @TruffleBoundary
    private void resize() throws OverflowException {
        handles = Arrays.copyOf(handles, PythonUtils.multiplyExact(handles.length, 2) - 1);
    }

    public void free(Node inliningTarget, HPyCloseHandleNode closeHandleNode) {
        assert cursor <= handles.length;
        for (int i = 0; i < cursor; i++) {
            closeHandleNode.execute(inliningTarget, handles[i]);
        }
        cursor = 0;
    }

    public void removeAll() {
        for (int i = 0; i < handles.length; i++) {
            handles[i] = null;
        }
        cursor = 0;
    }
}
